function[newb] = instantiate(b, domain, settings)
% INSTANTIATE replaces b with a newb that instantiates vals to settings

% Put a lot effort into doing this w/matlab functions instead of
% building it by iterating over the new matrix.  In the best case, we
% could do this in cost that is linear in the size of the new matrix
% with a small constant factor.  This implementation will take time
% that is roughly 3-4x the size of the original matrix.  Whether this
% is a good idea will depend on how efficient matlab's functions are
% in comparison to user produced ones.

if isempty(b.domain)
  newb = b;
  return;
end

% Figure out which variables we are keeping
[newb.domain, keep] = setdiff(b.domain, domain)

% The discards are the original set minus the keepers
discard = setdiff([1:length(b.domain)],keep)

% These are the dimensions of our original matrix
dims = size(b.values)

% Move the dimensions that will be removed to the front
ftemp = permute(b.values, [discard, keep])

% Now reshape the matrix so that the dimensions to be removed are
% collapsed into a single dimension.  This is useful b/c we can use
% matlab to pick a single entry in the first dimension and return the
% rest as a vector
ftemp = reshape(ftemp, [prod(dims(discard)) dims(keep)])

% Now we use the settings as in index into ftemp and reshape the rest
% to have the right dimensions
newb.values = reshape(ftemp(prod(settings),:), dims(keep))





